@@ -13,7 +13,7 @@ use Ref::Util qw( is_ref is_blessed_ref is_plain_coderef is_plain_scalarref );
 use Carp ();
 
 # ABSTRACT: Wasmtime instance class
-our $VERSION = '0.21'; # VERSION
+our $VERSION = '0.22'; # VERSION
 
 
 $ffi_prefix = 'wasm_instance_';
@@ -60,69 +60,154 @@ sub _cast_import
   Carp::croak("Non-extern object as import");
 }
 
-$ffi->attach( [ wasmtime_instance_new => 'new' ] => ['wasm_store_t','wasm_module_t','opaque[]','size_t','opaque*','opaque*'] => 'wasmtime_error_t' => sub {
-  my $xsub = shift;
-  my $class = shift;
-  my $module = shift;
-  my $store = is_blessed_ref($_[0]) && $_[0]->isa('Wasm::Wasmtime::Store')
-    ? shift
-    : Carp::croak('Creating a Wasm::Wasmtime::Instance instance without a Wasm::Wasmtime::Store object is no longer allowed');
+if(_v0_23_0())
+{
+  require FFI::Platypus::Memory;
 
-  my $ptr;
-  my @keep;
-
-  if(defined $_[0] && !is_ref($_[0]))
-  {
-    ($ptr) = @_;
-    return bless {
-      ptr    => $ptr,
-      module => $module,
-      keep   => \@keep,
-    }, $class;
-  }
-  else
-  {
-    my($imports) = @_;
-
-    $imports ||= [];
-    Carp::confess("imports is not an array reference") unless ref($imports) eq 'ARRAY';
-    my @imports = @$imports;
-    my $trap;
-
-    {
-      my @mi = @{ $module->imports };
-      if(@mi != @imports)
-      {
-        Carp::croak("Got @{[ scalar @imports ]} imports, but expected @{[ scalar @mi ]}");
-      }
-
-      @imports = map { _cast_import($_, shift @mi, $store, \@keep) } @imports;
-    }
+  $ffi->attach( [ wasmtime_instance_new => 'new' ] => ['wasm_store_t','wasm_module_t','record(Wasm::Wasmtime::Vec)*','opaque*','opaque*'] => 'wasmtime_error_t' => sub {
+    my $xsub = shift;
+    my $class = shift;
+    my $module = shift;
+    my $store = is_blessed_ref($_[0]) && $_[0]->isa('Wasm::Wasmtime::Store')
+      ? shift
+      : Carp::croak('Creating a Wasm::Wasmtime::Instance instance without a Wasm::Wasmtime::Store object is no longer allowed');
 
     my $ptr;
-    if(my $error = $xsub->($store, $module, \@imports, scalar(@imports), \$ptr, \$trap))
+    my @keep;
+
+    if(defined $_[0] && !is_ref($_[0]))
     {
-      Carp::croak("error creating module: " . $error->message);
+      ($ptr) = @_;
+      return bless {
+        ptr    => $ptr,
+        module => $module,
+        keep   => \@keep,
+      }, $class;
     }
     else
     {
-      if($trap)
+      my($imports) = @_;
+
+      $imports ||= [];
+      Carp::confess("imports is not an array reference") unless ref($imports) eq 'ARRAY';
+      my @imports = @$imports;
+      my $trap;
+
       {
-        $trap = Wasm::Wasmtime::Trap->new($trap);
-        die $trap;
+        my @mi = @{ $module->imports };
+        if(@mi != @imports)
+        {
+          Carp::croak("Got @{[ scalar @imports ]} imports, but expected @{[ scalar @mi ]}");
+        }
+
+        @imports = map { _cast_import($_, shift @mi, $store, \@keep) } @imports;
+      }
+
+      my $imports_vec = Wasm::Wasmtime::Vec->new(
+        size => scalar @imports,
+        data => scalar(@imports) > 0 ? do {
+          my $count = scalar @imports;
+          my $ptr = FFI::Platypus::Memory::malloc($ffi->sizeof('opaque') * $count);
+          # void *memcpy(void *dest, const void *src, size_t n)
+          FFI::Platypus->new( lib => [undef] )->function( 'memcpy' => [ 'opaque', "opaque[$count]", 'size_t' ] => 'opaque' )->call($ptr, \@imports, $ffi->sizeof('opaque') * $count);
+        } : undef,
+      );
+
+      my $ptr;
+      if(my $error = $xsub->($store, $module, $imports_vec, \$ptr, \$trap))
+      {
+        FFI::Platypus::Memory::free($imports_vec->data) if defined $imports_vec->data;
+        Carp::croak("error creating module: " . $error->message);
       }
       else
       {
-        return bless {
-          ptr    => $ptr,
-          module => $module,
-          keep   => \@keep,
-        }, $class;
+        FFI::Platypus::Memory::free($imports_vec->data) if defined $imports_vec->data;
+        if($trap)
+        {
+          $trap = Wasm::Wasmtime::Trap->new($trap);
+          die $trap;
+        }
+        else
+        {
+          return bless {
+            ptr    => $ptr,
+            module => $module,
+            keep   => \@keep,
+          }, $class;
+        }
       }
     }
-  }
 
-});
+  });
+
+}
+else
+{
+
+  $ffi->attach( [ wasmtime_instance_new => 'new' ] => ['wasm_store_t','wasm_module_t','opaque[]','size_t','opaque*','opaque*'] => 'wasmtime_error_t' => sub {
+    my $xsub = shift;
+    my $class = shift;
+    my $module = shift;
+    my $store = is_blessed_ref($_[0]) && $_[0]->isa('Wasm::Wasmtime::Store')
+      ? shift
+      : Carp::croak('Creating a Wasm::Wasmtime::Instance instance without a Wasm::Wasmtime::Store object is no longer allowed');
+
+    my $ptr;
+    my @keep;
+
+    if(defined $_[0] && !is_ref($_[0]))
+    {
+      ($ptr) = @_;
+      return bless {
+        ptr    => $ptr,
+        module => $module,
+        keep   => \@keep,
+      }, $class;
+    }
+    else
+    {
+      my($imports) = @_;
+
+      $imports ||= [];
+      Carp::confess("imports is not an array reference") unless ref($imports) eq 'ARRAY';
+      my @imports = @$imports;
+      my $trap;
+
+      {
+        my @mi = @{ $module->imports };
+        if(@mi != @imports)
+        {
+          Carp::croak("Got @{[ scalar @imports ]} imports, but expected @{[ scalar @mi ]}");
+        }
+
+        @imports = map { _cast_import($_, shift @mi, $store, \@keep) } @imports;
+      }
+
+      my $ptr;
+      if(my $error = $xsub->($store, $module, \@imports, scalar(@imports), \$ptr, \$trap))
+      {
+        Carp::croak("error creating module: " . $error->message);
+      }
+      else
+      {
+        if($trap)
+        {
+          $trap = Wasm::Wasmtime::Trap->new($trap);
+          die $trap;
+        }
+        else
+        {
+          return bless {
+            ptr    => $ptr,
+            module => $module,
+            keep   => \@keep,
+          }, $class;
+        }
+      }
+    }
+
+  });
+}
 
 
 sub module { shift->{module} }
@@ -156,7 +241,7 @@ Wasm::Wasmtime::Instance - Wasmtime instance class
 
 =head1 VERSION
 
-version 0.21
+version 0.22
 
 =head1 SYNOPSIS
 
@@ -241,7 +326,7 @@ Graham Ollis <plicease@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2020 by Graham Ollis.
+This software is copyright (c) 2020-2022 by Graham Ollis.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
